/*
SAS Microarray Affymetrix Input Engine

Name: AffymetrixInputEngine
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 28Aug02 initial coding   
   tzuchu 22Jan04 adding binary CEL, CHP files 
   tzuchu 05Mar04 separating from Input Engine

Description: InputEngine imports user provided 
raw file into SAS data set. The input data set 
is assumed to be Affymetrix CEL (probe data), 
or CHP (signal data) files.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

OPTIONS nospool;
* program start;
* define input parameters;
%let DataPlatform = CEL version 4; 
* Specify platform of data with one of the following options:
1. CEL version 3 (text format)
2. CEL version 4 (binary format)
3. CHP ; 

%let CDFInFile = ; 
* Affy CDF file with full path;

%let DataInPath = ; 
* path of data;

%let UploadRawFiles = No; 
* Specify "Yes" to upload Raw Files (in original format) into warehouse;

%let ExpInFile = ; 
* experimental design table;

%let ExpType = ; 
* file type of experimental design table. Excel Type is not accepted for 
Unix Server.;

%let ProbeUsed = PM; 
* option of Affy/CEL data for applying PM, difference (PM-MM), or pair 
(stack PM and MM); 

%let Log2Transform = Yes; 
* option to take logarithm based 2 transformation on raw data.;

%let ScanRows = 20; 
* The first number of rows in each raw data file to be scanned to determinte the
attributes of variables (columns) in the combined SAS data set.  The attributes 
include type (such as numerical, charcater, and date) and length of variables. 
Notice that once the attributes are decided, it will apply to entire data file. 
The length of character variable is main concern among attributes.  If there is
any row after the first "ScanRows" having larger length than the determined length
of variable, it will be truncated.;
 
%let OutPath = ; 
* Path of working library; 

%let OutData = ; 
* Name of output SAS data for further analysis. Length greater than 32 wil be 
truncated. Specified ProjectID will be applied as the name of output data when
leaving this field empty.;

%let MacroPath = ; 
* path of SAS macros;

%let KeepProcessInfo = No; 
* option to store process information;


*ProcessBody;

%macro AffyInputEngine;

* project identifier. ;
%let ProjectID = ; 

*initialization;
%let ProcessName = AffyInputEngine;

%global exiterror;
%let exiterror = 0;
%let clockrunning = 0;

* invoke macros;
data _null_;
   infile "&MacroPath.UtilityMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for utility macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.UtilityMacros.sas" / nosource;

data _null_;
   infile "&MacroPath.InputEngineMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for input engine macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.InputEngineMacros.sas" / nosource;

%CheckInputs;
%if &exiterror %then %goto exit;
%CheckForBlank(&DataPlatform,Platform of data);
%if &exiterror %then %goto exit;
%CheckForBlank(&CDFInFile,CDF file);
%if &exiterror %then %goto exit;

%let DataPlatForm = Affy/&DataPlatForm;
%let Gene = Unit;
%BeforeDataParse;
%if &exiterror %then %goto exit;
%if %upcase(%substr(&CDFInFile,%length(&CDFInFile)-8)) = .SAS7BDAT %then %do;
   %PathName(&CDFInFile);
   libname tmp "&TmpPath";   
   proc datasets lib=Work nolist;
      %if %sysfunc(exist(CDF)) %then delete CDF;;
      copy in=tmp out=Work;
      select &TmpName;
      change &TmpName=CDF;
   quit;
%end;
%else %do;
   %CDFParse;
%end;

%if %upcase(%substr(&DataPlatform,6,3)) = CEL %then %do;
   %let CelVersion = %substr(&DataPlatform,18,1);
   %CELParse;
%end;
%else %if %upcase(%substr(&DataPlatform,6,3)) = CHP %then %do;
   %CHPParse;
%end;
%else %do;
   %let exiterror = 1;
   %put ERROR: The specified data plateform does not exist.;
   %goto exit;
%end;

%AfterDataParse;
%if &exiterror %then %goto exit;
%Packaging;

%exit:;
%if (&syserr | &exiterror) %then %put ERROR: &ProcessName exited due to errors.;
%if (&clockrunning) %then %ClockStop(&ProcessName);
%ImportScanRows(Flag=0);
%mend AffyInputEngine;
%AffyInputEngine;