/*
SAS Microarray Solution Annotation Input Engine

Name: AnnotationAP
Author: Shibing Deng, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   shdeng 26Feb04 initial coding

Description: Annotation AP imports user provided annotation file into SAS data set.
Then output an unigue ID to Java program for querying annotation.  Annotation AP then read in
the table created by java and merge back with user supplied annotation file.
*/
OPTIONS nospool;

*ProcessVariables;
%let AnnoInFile= ; * Input file with full path. The file should have an Gene ID column with column name given in GeneIDVar below;

%let AnnoType= ; * Input file type. It must be one of the following:
	  1. TAB = Tab delimited text file,
      2. CSV = comma delimited text file,
      3. EXCEL = Excel file (Excel 2000 or later) (not accepted for Unix server),
      4. SAS = SAS dataset.;
%let GeneIDVar= ; *Column or Variable name of Gene Identifier.;
%let GeneIdentifier= ; * Gene Identifier. It must be one of the following: 
      1. GenBank Accession,
      2. UniGene Cluster Id,
      3. Locus Link Id,  
      4. Gene Symbol. 3 and 4 will be implemented in next release ;
%let Organism= ; * Organism. It must be one of the following:
	  1. Homo sapiens,
	  2. Mus musculus,
	  3. Rattus norvegicus,
	  4. Drosophila melanogaster.;

%let accSelected= TRUE; * true/false indicating whether user wants GenBank Accession as part of anno info;
%let clustTitleSelected= TRUE; * true/false indicating whether user wants UniGene Title as part of anno info;
%let clustIdSelected= TRUE; * true/false indicating whether user wants UniGene Cluster Id & link as part of anno info;
%let symbolSelected= TRUE; * true/false indicating whether user wants Gene Symbol & link as part of anno info;
%let locusIdSelected= FALSE; * true/false indicating whether user wants Locus Id & link as part of anno info;
%let entrezGeneSelected= FALSE; * true/false indicating whether user wants Entrez Gene link as part of anno info;
%let homolSelected= FALSE; * true/false indicating whether user wants HomoloGene link as part of anno info;
%let chromoSelected= FALSE; * true/false indicating whether user wants Chromosome Location as part of anno info;
%let cytoSelected= FALSE; * true/false indicating whether user wants Cytogenetic Location as part of anno info;
%let giSelected= FALSE; * true/false indicating whether user wants GI Number & link as part of anno info;
%let protIdSelected= FALSE; * true/false indicating whether user wants Protein Id & link as part of anno info;
%let cloneIdSelected= FALSE; * true/false indicating whether user wants Clone Id as part of anno info;
%let libIdSelected= FALSE; * true/false indicating whether user wants Library Id & link as part of anno info;
%let seqTypeSelected= FALSE; * true/false indicating whether user wants Sequence Type as part of anno info;
%let keggGeneSelected= FALSE; * true/false indicating whether user wants KEGG Gene link as part of anno info;
%let omimSelected = FALSE; * true/false indicating whether user wants OMIM Id & link as part of anno info;
%let swissSelected = FALSE; * true/false indicating whether user wants Swissprot & link as part of anno info;
%let ensembSelected= FALSE; * true/false indicating whether user wants Ensembl link as part of anno info;
%let nciSelected= FALSE; * true/false indicating whether user wants CGAP Pathway link as part of anno info;
%let goSelected= FALSE; * true/false indicating whether user wants Gene Ontology Term and link as part of anno info;
%let pubmedCitSelected = FALSE; * true/false indicating whether user wants Pubmed/Citations as part of anno info;
%let pubmedGeneSelected= FALSE; * true/false indicating whether user wants Pubmed/Gene Name as part of anno info;
%let pubmedTransgenSelected= FALSE; * true/false indicating whether user wants Pubmed/Transgene/Knockout as part of anno info;
%let entrezSelected= FALSE; * true/false indicating whether user wants Entrez link as part of anno info;
%let organismSelected= FALSE; * true/false indicating whether user wants Organism name as part of anno info;
%let htmlOneRowPerRecord=Multiple rows per gene; * Value indicating whether user selects one row per gene display or multiple row per gene display in Anno HTML file;
%let cloneEndSelected=FALSE; * true/false indicating whether user wants 3' or 5' End of clone info as part of anno info;
%let refSeqSelected=FALSE; * true/false indicating whether user wants reference sequence Accessions as part of anno info;
%let cgapGeneSelected=FALSE; * true/false indicating whether user wants CGAP Gene link as part of anno info;

%let AnnoOutData = ; * Name of annotation in SAS data format;
%let OutPath = ; * Path the converted SAS data stored;
%let MacroPath = ; * Path the UtilityMacros stored;
%let SDS_Create_Anno = true; * Hidden variable/field for SDS internal use;

%let ClientDownload = No;
* Download output data set to the client?  Specify Yes or No.  
The default is No. Specify Yes if you want to create a local 
copy for quicker viewing or saving. Be aware that specifying 
Yes with large data sets can result in long run times.;

%let KeepProcessInfo = No;
* option to keep process information.  If specify "Yes", a data set,
_processInfo, will be add to result tree;

*ProcessBody;

*Internal macro variable;
%let AnnoStep=1;*Step in creating Annotation file, value= 1 or 2;

%macro AnnoParse;
*initialization;
%let ProcessName = AnnotationAP;

%global exiterror;
%let exiterror = 0;
%let clockrunning = 0;
options nospool;
data _null_;
   infile "&MacroPath.UtilityMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for SAS macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.UtilityMacros.sas" / nosource;
%ClockStart(&ProcessName);
%if &syserr %then %goto exit;
%let clockrunning = 1;

%let outTxtFileName = %lowcase(&AnnoOutData).txt;
%let outHtmlFileName = %lowcase(&AnnoOutData).html;
%CheckDataName(_name=AnnoOutData,_default=annooutdata);

%CheckForBlank(&AnnoInFile,Annotation input File);
%if &exiterror %then %goto exit;
%PathName(&AnnoInFile);
%if &exiterror %then %do;
   %put ERROR: Annotation input dataset specification is invalid.;
   %goto exit;
%end;
%CheckForBlank(&AnnoType,Data type of the annotation input file);
%if &exiterror %then %goto exit;
%CheckExcel(&AnnoType,the annotation input file);
%if &exiterror %then %goto exit;
%let AnnoType=%upcase(&AnnoType);
%if &annotype=1 | &annotype=TABLE | &annotype=TXT %then %let annotype=TAB;
%if &annotype=2  %then %let annotype=CSV;
%if &annotype=3 | &annotype=EXCEL | &annotype=XLS %then %let annotype=EXCEL2000;
%if ^(&AnnoType = TAB | &AnnoType=CSV | &AnnoType=SAS | &AnnoType=EXCEL2000) %then %do;
	%put ERROR: Incorrect AnnoType.;
   %goto exit;
%end;
%CheckForBlank(&OutPath,Output path);
%if &exiterror %then %goto exit;
%CheckForBlank(&GeneIDVar,Output path);
%if &exiterror %then %goto exit;
%CheckDataName(_name=GeneIDVar,_default=accession);

%CheckForBlank(&AnnoOutData,Annotation Output Dataset);
%if &exiterror %then %goto exit;
%*if ^%length(&GeneIDVar) %then %let GeneIDVar=ACCESSION;/*default to accession*/

Libname AnnoLib "&OutPath";
%if &sysscp=WIN %then %let _slash=\;
%else %let _slash=/;

  *Second part;
%if %sysfunc(fexist(&outpath.&outTxtFileName)) %then %do;
	%put ERROR: No annotation results avaiable for import;
	%goto exit;
	%end;

  %let workpath=%sysfunc(pathname(work));
   proc registry export="&workpath.&_slash.registry.txt";
   run;
   %ImportScanRows(Flag=1,ScanRows=200);
   PROC IMPORT OUT= &AnnoOutData
        DATAFILE= "&outpath.&outTxtFileName"
        DBMS=TAB REPLACE;
        GETNAMES=YES;
   RUN;

   %if %sysfunc(fexist(registry)) %then %do;
    filename registry "&workpath.&_slash.registry.txt";
     proc registry import=registry;
      run;
      %let frc=%sysfunc(fdelete(registry));
   %end;

%NObsVars(&AnnoOutData);
%if (&nobs=0 | &nvars=0) %then %do;
   data annolib.exitedwitherrors;
      error=1;
   run;
   %put ERROR: Error(s) were encountered creating annotation output data set. No Data were imported;
   %let exiterror = 1;
   %goto exit;
%end;

   %let dsid = %sysfunc(open(&AnnoOutData));
   %let NewName = %sysfunc(varname(&dsid,1));
   %let crc = %sysfunc(close(&dsid));

   proc sort data=annolib.&AnnoOutData(rename=(&GeneIDVar=&NewName)) out=Anno;
      by &NewName;
   run;
   proc sort data=&AnnoOutData;
      by &NewName;
   run;
*Check the length of the merge var;
      %let dsid=%sysfunc(open(Anno));
      %let len1=%sysfunc(varlen(&dsid,%sysfunc(varnum(&dsid,&Newname))));
      %let rc=%sysfunc(close(&dsid));
   %let dsid=%sysfunc(open(&AnnoOutData));
      %let len2=%sysfunc(varlen(&dsid,%sysfunc(varnum(&dsid,&Newname))));
      %let rc=%sysfunc(close(&dsid));
*If not equal, set to equal. See s0259132;
%if &len1 ne &len2 %then %do;
  data &AnnoOutData;
  length &Newname $&len1;
  set &AnnoOutData;
  run;
%end;

   data annolib.&AnnoOutData;
      merge anno &AnnoOutData;
      by &NewName;
   run;

   filename GeneID "&outpath.GeneID4java.txt";
   %let filedel=%sysfunc(fdelete(GeneID));

* Packaging;
%PackageBegin(&ProcessName);

%PackageInsertDataset(Annotation file for &ProcessName, annolib, &AnnoOutData, &ClientDownload);
%if %upcase(&KeepProcessInfo) = YES %then %do;
   %PackageInsertDataset(process information for &ProcessName, annolib, _processinfo);
%end;

%PackageInsertFile( html from annotation java code, &OutPath, &outHtmlFileName,, text/html, yes);
%PackagePublish(&OutPath,to_requester);
%PackageEnd;

%exit:;
%if (&syserr | &exiterror) %then %put ERROR: &ProcessName exited due to errors.;
%if (&clockrunning) %then %ClockStop(&ProcessName);

%mend AnnoParse;
%AnnoParse;



