/*
SAS Scientific Discovery Solutions Data Import Engine

Name: DataImportEngine
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 03Mar04 initial coding                          

Description: DataImportEngine imports a user-
provided data file into a SAS data set by calling
Proc Import. The input data set is assumed to be
one of the types that the DBMS option in Proc 
Import can accept (including tab delimited, csv,
and Excel files).
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

OPTIONS nospool;

*ProcessVariables;
%let InFile = ; 
* Data file with full path;

%let FileType = ; 
* File type. Excel Type is not accepted for
Unix Server.;

%let UploadRawFile = No;
* Specify "Yes" to upload InFile (in original
format) into warehouse;

%let VarNameRow = 1;
* Specifying the row number which contents the variable 
(column) names. Since this row number will be applied
for all data files, this row is assumed to be identical
and contain the names of the SAS data set variables to be
created. The names must be unique and, if necessary, will 
be converted to valid SAS variables names with no embedded 
spaces or special characters. When specifying "0", no 
headers names are read from the raw data files, and the 
SAS variable names are assigned to be Var1 through Varn 
(or F1 through Fn when importing Excel file), where n is 
the number of columns in the raw data files. In this case, 
you must keep track of the order and content of the 
columns and specify Var1 through Varn in appropriate places
in your experimental design table.;

%let VarNameList= ;
* Specify the desired variable names to replace those used
by Proc Import. Separate names by an empty space and/or 
combine with ranges of variables with NO spaces before or 
after the hyphen such as "G1-G10". Apply "." to keep the 
name used by Proc Import. Specified names must follow the 
naming convention for SAS variables. The order of this 
list must match the order of columns in the InFile. For 
Example, specifying

ID . . Factor1-Factor3 

Will rename the first variable to "ID", keep the names for 
the second and third variables, and rename the forth, fifth,
and sixth variables to "Factor1", "Factor2", and "Factor3".  
If there are more than six variables in the data, the rest 
of them will remain as the names used by Proc Import.; 

%let DataStartRow = 2;
* The row in the raw data files to begin reading data.  
Specify this when there are unneeded rows (for example, 
comments or remarks) at the beginning of each raw data 
file.;

%let OutData = ; 
* Name of output in SAS data format. Length greater than
32 wil be truncated. Specified ProjectID will be applied
as the name of output data when leaving this field empty.;

%let OutPath = ; 
* Path where converted SAS data is stored;

%let MacroPath = ; 
* Path where UtilityMacros is stored;

%let ScanRows = 20; 
* The number of rows in each raw data file to be scanned
to determine the attributes of variables (columns) in 
the SAS data set. The attributes include type (such as 
numerical, character, and date) and length of variables. 
Notice that once the attributes are decided, they are 
applied to the entire data file. The length of character
variables is the main concern among attributes. If there
is any row after the "ScanRows" having larger length 
than the determined length of the variable, it will be 
truncated.; 

%let KeepProcessInfo = No;
* option to keep process information. If specify "Yes",
a data set, _processInfo, will be add to result tree; 

*ProcessBody;

%macro DataImport;
* Name for project;
%let ProjectID = ;

*initialization;
%let ProcessName = DataImportEngine;

%global exiterror;
%let exiterror = 0;
%let clockrunning = 0;
options nospool;
data _null_;
   infile "&MacroPath.UtilityMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for SAS macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.UtilityMacros.sas" / nosource;
data _null_;
   infile "&MacroPath.InputEngineMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for input engine macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.InputEngineMacros.sas" / nosource;
%ClockStart(&ProcessName);
%if &syserr %then %goto exit;
%let clockrunning = 1;
%CheckForBlank(&InFile, InFile);
%if &exiterror %then %goto exit;
%PathName(&InFile);
%let InPath = &TmpPath;
%let InFileName = &TmpName2; 
%if &exiterror %then %do;
   %put ERROR: The specification of InFile is invalid.;
   %goto exit;
%end;
%CheckForBlank(&FileType,Data type of the input file);
%if &exiterror %then %goto exit;
* Check Excel file and OS system;
%CheckExcel(&FileType,The input file);
%if &exiterror %then %goto exit;
%CheckForBlank(&OutPath,Output path);
%if &exiterror %then %goto exit;

%if &sysscp=WIN %then %let _slash=\;
%else %let _slash=/;

* define libnames;
Libname OutLib "&OutPath";
* check name for OutData;
%CheckDataName(_name=OutData,_default=&ProcessName);

%if %upcase(&FileType) = SAS %then %do;
   %PathName(&InFile);
   libname TmpLib "&TmpPath";

   data OutLib.&OutData;
      set Tmplib.&TmpName;
   run;
%end;
%else %do;
   %if &ScanRows>20 %then %do;
      %let workpath=%sysfunc(pathname(work));
      proc registry export="&workpath.&_slash.registry.txt";
      run;
      %ImportScanRows(Flag=1,ScanRows=&ScanRows);
   %end;
   * import raw data file;
   %let VarList = ;
   %skip_header_import_2(DataFile=%quote(&InFile),DataType=&FileType,
            OutSet=OutLib.&OutData,FirstRow=&DataStartRow,VarNameRow=&VarNameRow,
            GuessRows=&ScanRows,_varlist=VarList);
   * rename if needed;
   %if %length(&VarNameList) %then %do;
	  * extract each name from VarNameList which lists names to be replace;
	  %let i = 1;
	  %let j = 1;
      %do %while(%length(%scan(&VarNameList,&i,%str( ))));
         %let tmp&i = %scan(&VarNameList,&i,%str( ));
         %let hyp=%index(&&tmp&i,%str(-));
         %if &hyp>0 %then %do;
            %let fnumloc=%sysfunc(indexc(&&tmp&i,'0123456789'));
            %let prefix=%substr(&&tmp&i,1,%eval(&fnumloc-1));
            %let snumloc=%eval(%sysfunc(indexc(%substr(&&tmp&i,%eval(&hyp+1)),'0123456789'))+&hyp);
            %let num1=%substr(&&tmp&i,&fnumloc,%eval(&hyp-&fnumloc));
	        %let num2=%substr(&&tmp&i,&snumloc);
	        %do k = &num1 %to &num2;
	           %let TmpName&j = &prefix.&k; 
	           %let j = %eval(&j+1); 
	        %end;
         %end;
         %else %do; 
            %let TmpName&j = &&tmp&i; 
            %let j = %eval(&j+1);  
         %end;
         %let i = %eval(&i+1);
      %end;
	  %nObsVars(OutLib.&OutData);
	  proc datasets lib=OutLib nolist;
		 modify &OutData;	     
         %do i = 1 %to %sysfunc(min(%eval(&j-1),&nVars));
	        %if %scan(&VarList,&i) ne &&TmpName&i & &&TmpName&i ne . %then
               rename %scan(&VarList,&i) = &&TmpName&i;;
	     %end; 
	  quit;
   %end;

   %if %sysfunc(fexist(registry)) %then %do;
      filename registry "&workpath.&_slash.registry.txt";
      proc registry import=registry;
      run;
      %let frc=%sysfunc(fdelete(registry));
   %end;
%end;

%NObsVars(OutLib.&OutData);
%if (&nobs=0 | &nvars=0) %then %do;
   data OutLib.exitedwitherrors;
      error=1;
   run;
   %put ERROR: Error(s) were encountered creating output data set.;
   %let exiterror = 1;
   %goto exit;
%end;

* Packaging;
%PackageBegin(&ProcessName);
%PackageInsertDataset(data from &ProcessName,outlib,&OutData,No);
%if %upcase(&KeepProcessInfo) = YES %then %do;
   %PackageInsertDataset(process information of &ProcessName,outlib,_processinfo,No);
%end;
%if %upcase(&UploadRawFile)=YES %then %do;
   %if %upcase(&FileType) = TAB | %upcase(&FileType) = CSV %then %let mType = text/plain;
   %else %if %upcase(%substr(&FileType,1,5)) = EXCEL %then %let mType = application/xls;
   %else %let mType = application/binary;
   %PackageInsertFile(Raw Data,&InPath,&InFileName,,&mType,No);
%end;
%PackagePublish(&OutPath,to_requester);
%PackageEnd;

%exit:;
%if (&syserr | &exiterror) %then %put ERROR: &ProcessName exited due to errors.;
%if (&clockrunning) %then %ClockStop(&ProcessName);
%mend DataImport;
%DataImport;
