/*
SAS Scientific Discovery Solutions Analytical Process

Name: DataTranspose
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 23Jun03 initial coding                          

Description: 
DataTranspose transposes a SAS data set in 
"tall thin" format to one or both of two types, 
tall and wide. The tall data set is created 
according the orientation you specify using 
ColVars and RowVars.  The wide data set is 
a transpose of the tall data set. Typically, 
for microarray data, variables associated 
with arrays (experimental factors, array 
identifier) are specified in ColVars, 
whereas variable(s) associated with 
genes (gene identifier) are specified 
in RowVars. 

If there are multiple observations of each 
combination of RowVars, the mean of those 
observations will be calculated to represent 
the intensity of the associated RowVars.
This approach is useful for averaging probes 
within a probe set for Affymetrix GeneChip data 
and replicate spots (technical replicates) of 
single genes within arrays for cDNA array data. 
Using the stdresiduals data set from 
MixedModelAnalysis as input and specifying 
the variable Pred as the IntensityVar produces 
model-based summary intensities. Be aware that 
the wide data set can become unwieldy when there 
are thousands of genes. In this case, it can be 
more efficient to first use DataMerge to create 
a subset of the data.  Finally, annotation 
information is only included with the tall data set.
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%let InData = ; 
* Full path and name to the input SAS data set. It must be in "tall 
thin" format. For microarray data as an example, all of the expression
measurements are stacked into one variable, and additional variables 
distinguish arrays, array groups, channels, genes, and other aspects of
the experiment. The data set name must be lowercase.;

%let TransposeVar = ;
* Variable to be transposed.;

%let ColVars = ; 
* Unique combinations of the values of these variables will form the 
columns in the tall data set and the rows of the wide data set.;

%let RowVars = ; 
* Unique combinations of the values of these variables will form the 
rows in the tall data set and the columns of the wide data set. This 
specification must be consistent with what you specified for ColVars, 
otherwise an error message will be printed in the SAS log.;

%let TallDataName = ; 
* Name of tall data set.  It will not be created if you leave this field 
empty.;

%let WideDataName = ;
* Name of wide data set.  It will not be created if you leave this field 
empty.;

%let AnnoInData = ;  
* Annotation SAS data set, including the full path and name with final
extension. It must contain the AnnoMergeVar variable since the process 
uses this variable as a merge key. The data set name must be lowercase.;

%let AnnoMergeVar = ; 
* Variable used for merging annotation data.  If present, it must be one 
of the RowVars.;

%let OutPath = ; 
* Path for output on your SAS server. If you leave this blank, the system 
will use a default location.;

%let MacroPath = ; 
* Path for SAS macros.;

%let ClientDownload = No;
* Download output data set to the client?  Specify Yes or No.  
The default is No. Specify Yes if you want to create a local 
copy for quicker viewing or saving. Be aware that specifying 
Yes with large data sets can result in long run times.;

%let KeepProcessInfo = No;
* option to keep process information.  If specify "Yes", a data set, 
_processInfo, will be add to result tree;


*ProcessBody;

%macro DataTranspose;

*initialization and argument checking;
%let ProcessName = datatranspose; 
%global exiterror;
%let exiterror = 0;
%let clockrunning = 0;
options nospool;
data _null_;
   infile "&MacroPath.UtilityMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for SAS macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.UtilityMacros.sas" / nosource;
%ClockStart(&ProcessName);
%if &syserr %then %goto exit;
%let clockrunning = 1;
%CheckForBlank(&InData,Input dataset);
%if %sysfunc(fileexist(&InData.*)) = 0 %then %do; 
   %let exiterror = 1;
   %put ERROR: Specified InData does not exist.;
%end;
%PathName(&InData);
%if &exiterror %then %do;
   %put ERROR: Input dataset specification is invalid.;
   %goto exit;
%end;
%let InData = &TmpName;
libname InLib "&TmpPath";
%CheckForBlank(&OutPath,Output path);
libname OutLib "&OutPath"; 
%CheckForBlank(&TransposeVar,Transpose variable);
%CheckForBlank(&ColVars,Column variables);
%CheckForBlank(&RowVars,Row variables);
%if &TallDataName= & &WideDataName= %then %do;
   %let exiterror = 1;
   %put ERROR: At least one data name must be specified.;
%end;
%if &exiterror %then %goto exit;
%CheckVarExist(&TransposeVar,InLib.&InData,Transpose variable);
%CheckVarExist(&ColVars,InLib.&InData,Column variables);
%CheckVarExist(&RowVars,InLib.&InData,Row variables);
%if &exiterror %then %goto exit;
%if %length(&AnnoInData) %then %do; 
   %CheckForBlank(&AnnoMergeVar,AnnoMergeVar variable);  
   %CheckVarExist(&AnnoMergeVar,InLib.&InData,AnnoMergeVar variable);
   %if %sysfunc(fileexist(&AnnoInData)) = 0 %then %do; 
      %let exiterror = 1;
      %put ERROR: Specified AnnoInData does not exist.;
   %end;
   %PathName(&AnnoInData);
   %if &exiterror %then %do;
      %put ERROR: Annotation dataset specification is invalid.;
      %goto exit;
   %end;
   %let AnnoData = &TmpName;
   libname AnnoLib "&TmpPath";
   %CheckVarExist(&AnnoMergeVar,AnnoLib.&AnnoData,AnnoMergeVar variable);
   %if &exiterror %then %goto exit;
%end;

%if %length(&TallDataName) %then %ClockStart(Tall data set,Initiate=0);
%else %ClockStart(Wide data set,Initiate=0);
* create column variable list;
%let CVCon = trim(left(%scan(%quote(&ColVars),1)));
%let i = 2;
%do %while(%length(%scan(%quote(&ColVars),&i)));
   %let CVCon = &CVCon || "_" || trim(left(%scan(%quote(&ColVars),&i)));
   %let i = %eval(&i+1);
%end; 

* create row variable list;
%let RVCon = trim(left(%scan(%quote(&RowVars),1)));
%let i = 2;
%do %while(%length(%scan(%quote(&RowVars),&i)));
   %let RVCon = &RVCon || "_" || trim(left(%scan(%quote(&RowVars),&i)));
   %let i = %eval(&i+1);
%end; 

proc sort data=InLib.&InData out=InData;
   by &RowVars &ColVars;
run;

/* this step can be avoided if there is only one obs per gene per array */
proc means data=InData mean noprint;
   by &RowVars &ColVars;
   var &TransposeVar;
   output out=NewInData Mean=&TransposeVar;
run;

data NewInData;
   set NewInData;
   CID = &CVCon;
   RID = &RVCon;
   drop _type_ _freq_;
run;
 
%let Label = 0;
%let tVar = &TransposeVar; 
%CheckLabel(NewInData,tVar,New=Label);
%if %upcase(&Label)=%upcase(&tVar) %then %let d_label = _label_; 
%else %let d_label = ;

%if %length(&TallDataName) %then %do;
   %CheckDataName(_name=TallDataName);
   proc transpose data=NewInData out=OutLib.&TallDataName(drop=_name_ &d_label) Prefix=&TransposeVar._;
      by &RowVars;
      var &TransposeVar;
      id CID;
   run;

   %if %length (&AnnoInData) %then %do;
      proc sort data=AnnoLib.&AnnoData out=AnnoS;
         by &AnnoMergeVar;
      run;

      proc sort data=OutLib.&TallDataName;
         by &AnnoMergeVar;
      run;

      data OutLib.&TallDataName;
         merge AnnoS(in=in_1) OutLib.&TallDataName(in=in_2);
	     by &AnnoMergeVar;
	     if in_1 & in_2;
      run;
   %end; 

   %if &syserr %then %do;
      %put ERROR: Unable to create tall dataset.;
      %goto exit;
   %end;
   %ClockStop(Tall data set);
   %if %length(&WideDataName) %then %ClockStart(Wide data set,Initiate=0);
%end;

%if %length(&WideDataName) %then %do;
   %CheckDataName(_name=WideDataName);
   * transpose data to sample (ColVars) as row;
   proc sort data=NewInData;
      by &ColVars &RowVars;
   run;

   proc transpose data=NewInData out=OutLib.&WideDataName(drop=_name_ &d_label) Prefix=&TransposeVar._;
      by &ColVars;
      var &TransposeVar;
      id RID;
   run;

   %if &syserr %then %do;
      %put ERROR: Unable to create wide data set.;
      %goto exit;
   %end;
   %ClockStop(Wide data set);
%end;

* Packaging;
%PackageBegin(&ProcessName);
%if %length(&TallDataName) %then
%PackageInsertDataset(Tall data from &ProcessName, outlib, &TallDataName, &ClientDownload);
%if %length(&WideDataName) %then
%PackageInsertDataset(Wide data from &ProcessName, outlib, &WideDataName, &ClientDownload);
%if %upcase(&KeepProcessInfo) = YES %then %do;
   %PackageInsertDataset(process information for &ProcessName, outlib, _processinfo);
%end;
%PackagePublish(&OutPath,to_requester);
%PackageEnd;

%exit:;
%if (&syserr | &exiterror) %then %put ERROR: &ProcessName exited due to errors.;
%if (&clockrunning) %then %do;
   %ClockStop(&ProcessName);
%end;

%mend DataTranspose;

* run the macro;
%DataTranspose;

