/*
SAS Scientific Discovery Solutions Input Engine

Name: ExperimentInputEngine
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 28Aug02 initial coding   
   tzuchu 05Mar04 separate from Previous Input Engine 

Description: ExperimentInputEngine imports user 
provided raw file from table formated (text 
delimited, EXCEL) or SAS data sets into SAS data 
set with combining associated experimental 
information provided. When applying this Input
Engine, all the data files are assumed to have 
same structure, such as the same order of column 
and the same starting data row.
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

options nospool;
* program start;
%let DataInPath = ; 
* path of data;

%let DataType = ; 
* Type of raw data. Specify CSV, Excel97, Excel2000, SAS,
or Tab. Excel Type is not accepted for Unix Server.;

%let VarNameRow = 1;
* Specifying the row number which contents the variable 
(column) names. Since this row number will be applied
for all data files, this row is assumed to be identical
and contain the names of the SAS data set variables to be
created. The names must be unique and, if necessary, will 
be converted to valid SAS variables names with no embedded 
spaces or special characters. When specifying "0", no 
headers names are read from the raw data files, and the 
SAS variable names are assigned to be Var1 through Varn 
(or F1 through Fn when importing Excel file), where n is 
the number of columns in the raw data files. In this case, 
you must keep track of the order and content of the 
columns and specify Var1 through Varn in appropriate places
in your experimental design table.;

%let DataStartRow = 2;
* The row in the raw data files to begin reading data. 
Specify this when there are unneeded rows (for example,
comments or remarks) at the beginning of each raw data 
file.;

%let UploadRawFiles = No; 
* Specify "Yes" to upload Raw Files (in original 
format) into warehouse;

%let ExpInFile = ; 
* experimental design table. The name(s) for raw 
data file(s) must include file extensions.;

%let ExpType = ; 
* file type of experimental design table. Excel Type 
is not accepted for Unix Server.;

%let IDVars = ; 
* Variable applied to identify observation. Typically,
they will be gene ID and block ID for microarray data.
Separate multiple variables by space. If space appears
within a single variable in the raw data file, please
substitute it by "_".;

%let Log2Transform = Yes; 
* option to take logarithm based 2 transformation 
on raw data.;

%let ScanRows = 20; 
* The first number of rows in each raw data file to 
be scanned to determinte the attributes of variables
(columns) in the combined SAS data set. The attributes 
include type (such as numerical, charcater, and date) 
and length of variables. Notice that once the 
attributes are decided, it will apply to entire data 
file. The length of character variable is main concern
among attributes. If there is any row after the first 
"ScanRows" having larger length than the determined 
length of variable, it will be truncated.; 

%let OutPath = ; 
* Path of output folder. A temperary folder (work 
library) will be applied if Leaving empty.; 

%let OutData = ; 
* SAS data for further analysis. Length greater than
32 wil be truncated. Specified ProjectID will be 
applied as the name of output data when leaving this
field empty.;

%let MacroPath = ; 
* path of sas macros;

%let KeepProcessInfo = No; 
* option to store process information;


*ProcessBody;

%macro TableIE;
* project identifier;
%let ProjectID = ; 

*initialization;
%let ProcessName = ExperimentInputEngine;

%global exiterror;
%let exiterror = 0;
%let clockrunning = 0;

* invoke macros;
data _null_;
   infile "&MacroPath.UtilityMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for utility macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.UtilityMacros.sas" / nosource;

data _null_;
   infile "&MacroPath.InputEngineMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for input engine macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.InputEngineMacros.sas" / nosource;

* adjust for SAS naming convention if IDVars is not a valid SAS variable name;
%if %length(&IDVars) %then %do;
   data tmp; 
      %let i = 1;
	  %let AllID = ;
      %do %while(%length(%scan(&IDVars,&i)));
         ID&i = "%scan(&IDVars,&i)";
         %let AllID = &AllID ID&i;  
		 %let i = %eval(&i+1);
	  %end;
	  %let nID = %eval(&i-1);
   run;
   %naming(Dset=tmp,nVars=&nID,AllVars=&AllID);
   data _null_; 
      set tmp; 
	  %do j = 1 %to &nID;
         call symput("ID&j",ID&j); 
	  %end;
   run;
   %let IDVars = ;
   %do j = 1 %to &nID;
      %let IDVars = &IDVars &&ID&j;
   %end;
%end;
%let DataPlatform = ExperimentInputEngine;

%CheckInputs;
%if &exiterror %then %goto exit;
%CheckForBlank(&IDVars,Variable distinguishing observations);
%if &exiterror %then %goto exit;
%CheckForBlank(&DataType,Data type of raw data);
%if &exiterror %then %goto exit;
* Check Excel file and OS system;
%CheckExcel(&DataType,The input raw data);
%if &exiterror %then %goto exit;
%BeforeDataParse;
%if &exiterror %then %goto exit;
%TableParse;
%AfterDataParse;
%if &exiterror %then %goto exit;
%Packaging;

%exit:;
%if (&syserr | &exiterror) %then %put ERROR: &ProcessName exited due to errors.;
%if (&clockrunning) %then %ClockStop(&ProcessName);
%ImportScanRows(Flag=0);

%mend TableIE;
%TableIE;
