/*
SAS Scientific Discovery Solutions Input Engine

Name: InputEngine
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 28Aug02 initial coding   
   tzuchu 22Jen04 adding binary CEL, CHP files 

Description: InputEngine imports user provided raw file into SAS data set.  
The input data set is assumed to be a text delimited, EXCEL, ScanAlyze output,
Affymetrix CEL (probe data), and Affymetrix CHP (signal data) files.
*/

OPTIONS nospool;
* program start;
* define input parameters;
%let ProjectID = ; * project identifier;
%let DataPlatform = ; * platform of data; 
%let CelVersion = ; * Format version of CEL files. Input Engine supports verion 3 
(ASCII text format) and version 4 (binary format).;
%let DataPath = ; * path of data;
%let ExpFile = ; * experimental design table;
%let ExpType = ; * file type of experimental design table;
%let MacroPath = ; * path of sas macros;

%let CDFFile = ; * Affy CDF file (for Affy/CEL or Affy/CHP);
%let ProbeUsed = PM; * option of Affy/CEL data for applying PM, difference(PM-MM), 
or pair(stack PM and MM); 
%*let KeepCoordinates = ; * includeing X-, Y- coordinates in data.  This is Required
if applying Pseudo Image AP; 
%let GeneVar = ; * Variable distinguishing genes (for table formatted only);
%let Log2Transform = Yes; * option to take logarithm based 2 transformation 
on raw data.;

%let ScanRows = 20; 
* The first number of rows in each raw data file to be scanned to determinte the
attributes of variables (columns) in the combined SAS data set.  The attributes 
include type (such as numerical, charcater, and date) and length of variables. 
Notice that once the attributes are decided, it will apply to entire data file. 
The length of character variable is main concern among attributes.  If there is
any row after the first "ScanRows" having larger length than the determined length
of variable, it will be truncated.   

* define output parameters;
%let OutPath = ; * Path of working library; 
%let OutData = ; * SAS data for further analysis;

%*let ProcessInfo = No; * option to store process information;


/* Example 1: Gibson's fly data; 
%let ProjectID = Drosophila;
%let DataPlatform = ScanAlyze;
%let DataPath = C:\RDMData\Warehouses\Drosophila\Data\;
%let ExpFile = C:\RDMData\Warehouses\Drosophila\Data\AgingExperimentTable.xls;
%let ExpType = EXCEL2000;
%let OutData = AllDATs;
%let OutPath = C:\RDMData\Warehouses\Drosophila\Data\;
%let MacroPath = C:\RDMData\MacroLib\;
*/

/* Example 2: Trimmed Affy Latin Square (Human U95A);
%let projectID = AHLS;
%let DataPlatform = Affymetrix\Cel;
%let DataPath = C:\RDMData\Warehouses\Affymetrix\Data\; 
%let ExpFile = C:\RDMData\Warehouses\Affymetrix\Data\DesignTable.xls;
%let ExpType = EXCEL2000;
%let CDFFile = C:\RDMData\Warehouses\Affymetrix\Data\HG_U95A_trim.CDF; 
%let OutPath = C:\RDMData\Warehouses\Affymetrix\Data\; 
%let OutData = AHLSInput;
%let MacroPath = C:\RDMData\MacroLib\;
*/


*ProcessBody;

%macro InputEngine;

*initialization;
%let ProcessName = InputEngine;

%global exiterror;
%let exiterror = 0;
%let clockrunning = 0;
options nospool;

* invoke macros;
data _null_;
   infile "&MacroPath.UtilityMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for utility macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.UtilityMacros.sas" / nosource;

data _null_;
   infile "&MacroPath.InputEngineMacros.sas";
run;
%if &syserr %then %do;
   %put ERROR: Path for input engine macros is invalid.;
   %goto exit;
%end;
%include "&MacroPath.InputEngineMacros.sas" / nosource;

%ClockStart(&ProcessName);
%if &syserr %then %goto exit;
%let clockrunning = 1;

%CheckForBlank(&DataPlatform,Platform of data);
%if &exiterror %then %goto exit;
%CheckForBlank(&DataPath,Full path of data);
%if &exiterror %then %goto exit;
%CheckForBlank(&ExpFile,Experimental design table);
%if &exiterror %then %goto exit;
%CheckForBlank(&ExpType,Data type of Experimental design table);
%if &exiterror %then %goto exit;
%CheckForBlank(&OutPath,Output path);
%if &exiterror %then %goto exit;

* create output libaray;
libname OutLib "&OutPath";

%let workpath=%sysfunc(pathname(work));
proc registry export="&workpath.\registry.txt";
run;
%ImportScanRows(200);

%DesignParse;

%if %upcase(&DataPlatform) = SCANALYZE %then %do;
   %let Gene = Spot;
   %DATParse;
%end;
%if %upcase(&DataPlatform) = GENEPIX %then %do;
   %let Gene = ID;
   %DATParse;
%end;
%if %upcase(&DataPlatform) = AGILENT %then %do;
   %let Gene = FeatureNum;
   %DATParse;
%end;
%if %upcase(&DataPlatform) = AFFYMETRIX\CEL %then %do;
   %CheckForBlank(&CDFFile,CDF file);
   %if &CelVersion ne 3 & &CelVersion ne 4 %then %do;
      %let exiterror = 1; %put cel= &celversion;
      %put ERROR: Please specify 3 (text) or 4 (binary) for Cel version. ;
   %end;
   %if &exiterror %then %goto exit;
   %let Gene = Unit;
   %CDFParse;
   %CELParse;
%end;
%if %upcase(&DataPlatform) = AFFYMETRIX\CHP %then %do;
   %CheckForBlank(&CDFFile,CDF file);
   %let Gene = Unit;
   %CDFParse;
   %CHPParse;
%end;
%if %upcase(%substr(&DataPlatform,1,5)) = TABLE %then %do;
   %CheckForBlank(&GeneVar,Variable distinguishing genes);
   %if &exiterror %then %goto exit; 
   * adjust for SAS naming convention if GeneVar is not a valid SAS variable name;
   data tmp; GeneVar = "&GeneVar"; run;
   %naming(Dset=tmp,nVars=1,AllVars=GeneVar);
   data _null_; set tmp; call symput('Gene',GeneVar); run;
   %TableParse;
%end;

%NObsVars(OutLib.&OutData);
%if %sysfunc(exist(OutLib.exitedwitherrors)) %then %do;
   proc datasets lib=OutLib nolist; 
      delete exitedwitherrors; 
   run;
%end;
%if (&nobs=0 | &nvars=0) %then %do;
   data OutLib.exitedwitherrors;
      error=1;
   run;
   %put ERROR: Error(s) were encountered creating output combining data set.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %sysfunc(fexist(registry)) %then %do;
   filename registry "&workpath.\registry.txt";
   proc registry import=registry;
   run;
   %let frc=%sysfunc(fdelete(registry));
%end;

%exit:;
%if (&syserr | &exiterror) %then %put ERROR: &ProcessName exited due to errors.;
%if (&clockrunning) %then %ClockStop(&ProcessName);
filename registry "&workpath.\registry.txt";
%if %sysfunc(fexist(registry)) %then %do;
   proc registry import=registry;
   run;
   %let frc=%sysfunc(fdelete(registry));
%end;

%mend InputEngine;
%InputEngine;