/*
SAS Scientific Discovery Solutions 

Name: UtilityMacros
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 28Aug02 initial coding
	sasrdw 27Jun03 upgraded publish macros

Description: UtilityMacros contains a collection 
of useful SAS macros that can be called from 
within analytical processes.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

* check for zero length argument;
%macro CheckForBlank(arg,label); 
   %if %length(&arg) = 0 %then %do;
      %put ERROR: &label specification is required.;
      %let exiterror = 1;
   %end;
%mend CheckForBlank;

* check for existence of variables;
%macro CheckVarExist(Vars,Dset,Label,ShowError=1);
   %let error = 0;
   %let count = 0;
   %let i = 1; 
   %do %while(%length(%scan(%quote(&Vars),&i)));
      %let found = 0; 
	  %let ds = %sysfunc(open(&Dset));
	  %let n = %sysfunc(attrn(&ds,NOBS)); 
	  %do j = 1 %to &n;
         %let rc=%sysfunc(fetchobs(&ds,&j));
         %let found = %sysfunc(varnum(&ds,%scan(%quote(&Vars),&i)));
		 %if &found %then %let j = &n;
	  %end;
      %let rc=%sysfunc(close(&ds));
	  %if ^&found %then %do;
         %let exiterror = 1;
		 %let error = 1;
		 %if &count %then %let BadVars = &BadVars,%scan(%quote(&Vars),&i);
		 %else %let BadVars = %scan(%quote(&Vars),&i);
		 %let count = %eval(&count+1);
	  %end;
      %let i = %eval(&i+1);
   %end; 
   %if &ShowError %then %do;
      %if &count>1 %then %let d = do;
      %else %let d = does;
      %if &error %then %put ERROR: &Label &BadVars &d not exist in the data set specified in &Dset.;
   %end;
%mend CheckVarExist;

* Check if variable &old in dataset &Dset has label.;
* If labe exists then replace &old (or asign &new) by label.; 
%macro CheckLabel(Dset,Old,New=);
   %let dsid = %sysfunc(open(&dset));
   %if &dsid %then %do;
      %if %sysfunc(varnum(&dsid,&&&old)) %then %do;
	     %let l = %sysfunc(varlabel(&dsid,%sysfunc(varnum(&dsid,&&&old))));
		 %if %length(&l) %then %do;
		    %if %length(&new) %then %let &new = &l;
			                  %else %let &old = &l;
		 %end;
		 %else %if %length(&new) %then %let &new = &old;
	  %end;
	  %else %put ERROR: Variable &old does not contain in dataset &dset.; 
      %let rc = %sysfunc(close(&dsid));
   %end;   
   %else %do;
      %put ERROR: Open for data set &dset failed when checking label status - %sysfunc(sysmsg());
   %end;
%mend CheckLabel;

* check return code for a function;
%macro Checkrc(function,_rc=);
   %if %length(&_rc) %then %let rc = &_rc;
   %if &rc = 0 %then %do;
      %put &function succeeded.;
   %end;
   %else %do;
      %let msg = %sysfunc(sysmsg());
      %put ERROR: &function failed - &msg;
   %end;
%mend Checkrc;

* check annotation parameters;
%macro CheckAnno(checkindata=1);
   %if %length(&AnnoInData) %then %do; 
   	%if %sysfunc(fileexist(&AnnoInData)) = 0 %then %do; 
   		%let exiterror = 1;
      	%put ERROR: Specified AnnoInData does not exist.;
			%goto caexit;
		%end;
   	%PathName(&AnnoInData);
   	%if &exiterror %then %do;
      	%put ERROR: Annotation data set specification is invalid.;
      	%goto caexit;
   	%end;
	%global AnnoData;
   	%let AnnoData = &TmpName;
	libname AnnoLib "&TmpPath";
   	%CheckForBlank(&AnnoMergeVar,AnnoMergeVar);
   	%if (&checkindata=1) %then %do;
     %CheckVarExist(&AnnoMergeVar,InLib.&InData,AnnoMergeVar);
    %end;
   	%CheckVarExist(&AnnoMergeVar,AnnoLib.&AnnoData,AnnoMergeVar);
		%if %length(&AnnoAccession) %then %do;
   		%CheckVarExist(&AnnoAccession,AnnoLib.&AnnoData,AnnoAccession);
		%end;
		%else %put NOTE: Functions using annotation accession numbers are not included in this analysis.;
   	%if %length(&AnnoLabel) %then %do;
			%CheckVarExist(&AnnoLabel,AnnoLib.&AnnoData,AnnoLabel);
		%end;
		%else %put NOTE: Annotation labels are not included in this analysis.;
   	%if &exiterror %then %goto caexit;
	%end;
	%else %do;
	   %put NOTE: Annotation information is not included in this analysis.;
		%let AnnoMergeVar = ;
	   %let AnnoAccession = ;
	   %let AnnoLabel = ;
	%end;
   %caexit:;
%mend CheckAnno;

* Check if user provided data name is valid a SAS data set name;
%macro CheckDataName(_name=,_default=);
   %if ^%length(&&&_name) %then %let &_name = %trim(%left(%lowcase(&_default)));
   data _null_;
      name = lowcase(trim(left("&&&_name")));
	  do i = 1 to length(name);
	     if substr(name,i,1) ^in ('0','1','2','3','4','5','6','7','8','9',
                                  'a','b','c','d','e','f','g','h','i','j',
	                              'k','l','m','n','o','p','q','r','s','t',
	 				              'u','v','w','x','y','z') then substr(name,i,1)=' ';
	  end;
	  name = translate(compbl(name),'_','');
	  if substr(name,1,1) in ('0','1','2','3','4','5','6','7','8','9') then do;
         name2='_'||name;
	     call symput("&_name",substr(name2,1,min(length(name2),32)));
	  end;
	  else call symput("&_name",substr(name,1,min(length(name),32)));
   run;
%mend CheckDataName;


%macro CheckByVars(_OldList=,_NewList=,_nVar=tmp);
   %if %length(&_OldList) %then %do;
      %let &_NewList = ;
	  %let &_nVar = 0;
      %let i = 1; 
	  %do %while(%length(%scan(&_OldList,&i,' ')));
	     %if %upcase(%scan(&_OldList,&i,' '))^=DESCENDING %then %do;
		    %let &_NewList = &&&_NewList %scan(&_OldList,&i,' ');
		    %let &_nVar = %eval(&&&_nVar+1);
		 %end;
		 %let i = %eval(&i+1);
	  %end;
   %end;
%mend CheckByVars;


* Check Excel file and OS system;
%macro CheckExcel(Type,ErrMsge,OS=WIN);
   %if (%length(&Type)>3) & (&sysscp ^= &OS) %then %do;
      %put ERROR: Excel type of &ErrMsge can not be imported by Unix server.; 
	  %let exiterror = 1;
   %end;
%mend CheckExcel;

* start timing;
%macro ClockStart(ProcessName,Initiate=1);
   libname outlib "&OutPath";
   data tmp;
      format Process $32. StartingDay date7. StartingTime time11.2 TotalTime time11.2 Status $10. Notes $100.;
      Process = "&processName";
	  StartingDay = round( date() );
	  StartingTime = round( time(),.01 );
	  TotalTime = .;
	  Status = "Processing";
	  Notes = " ";
      put "&ProcessName started on "  StartingDay date7. " " StartingTime time11.2 ".";
   run;
   %if &Initiate %then %do;
      data outlib._processinfo;
	     set tmp;
	  run;
   %end;
   %else %do;
      data outlib._processinfo;
	     set outlib._processinfo tmp;
	  run;
   %end;
%mend ClockStart;


* stop timing;
%macro ClockStop(ProcessName,Notes=" ");
   data tmp;
      set outlib._processinfo;
	  Obs = _n_;
	  if TotalTime = .;
   run;

   data tmp;
      set tmp end=eof;
	  if eof then call symput('Current',Obs); 
   run;

   data outlib._processinfo;
      set outlib._processinfo;
	  if _n_ = &Current Then do;
	     Day = round( date() );
	     Time = round( time(),.01 );
         if StartingDay = Day then TotalTime = round(Time-StartingTime,.01);
	     else TotalTime = (Day-StartingDay)*86400 + round(Time-StartingTime,.01);
		 Status = "Processed";
		 Notes = &Notes;
		 %if &syserr %then Notes = Notes || 'error(s) occur';
         put "&ProcessName finished on "  Day date7. " " Time time11.2 ".";
         put "The total time for &ProcessName was " Totaltime time11.2 ".";
	  end;
	  drop Day Time;
   run;
%mend ClockStop;

* make a variable list using proc contents;
%macro MakeVarList(data=,nvar=,prefix=);
   proc contents data=&data out=tmp(keep=name varnum) noprint; 
   run;
   data tmp;
      set tmp;
      if substr(NAME,1,%length(&prefix)) = "&prefix";
   run;
   %global &nvar;
   %let &nvar = 0;
   data _null_; 
      set tmp nobs=eof; 
      call symput("&nvar",trim(left(eof))); 
   run; 
   %if (&&&nvar = ) %then %let &nvar = 0;
   %let ni = &&&nvar;
   %if (&ni = 0) %then %do;
      %put NOTE: MakeVarList(data=&data,prefix=&prefix) has zero elements.;
   %end;
   %else %do;
      proc sort data=tmp;
	     by varnum;
      run;
      %do i = 1 %to &ni;
         data _null_; 
            set tmp; 
            if _n_ = &i; 
            call symput('tmp',name);
         run;
         %global &prefix.&i;
         %let &prefix.&i = &tmp;
	  %end;
   %end;
%mend MakeVarList;

* determine number of observations and variables;
%macro NObsVars(ds);
   %global nvars nobs;
   %let dset=&ds;
   %let dsid = %sysfunc(open(&dset));
   %if &dsid %then %do;
      %let nobs = %sysfunc(attrn(&dsid,NOBS));
      %let nvars = %sysfunc(attrn(&dsid,NVARS));
      %let rc = %sysfunc(close(&dsid));
   %end;
   %else %do;
      %put ERROR: Open for data set &dset failed - %sysfunc(sysmsg());
	  %let nobs = 0;
	  %let nvars = 0;
	  %let exiterror = 1;
   %end;
%mend NObsVars;

* Packaging macros;
%macro PackageBegin(ProcessName);
   %global ProcessID ProcessNameValue rcp;
   %let ProcessID = 0;
   %let description = &ProcessName Results Package;
   %let ProcessNameValue = Project=&ProcessName;
   %let rcp = 0;
   %syscall package_begin(ProcessID,description,ProcessNameValue,rcp);
   %Checkrc(Package_Begin,_rc=&rcp);
%mend PackageBegin;

/* for macros below, clientdownload indicates whether the file should be downloaded
   to the client, valid values are yes or no */
%macro PackageInsertDataset(description,libname,memname,clientdownload);
   %if ^%length(&clientdownload) %then %let clientdownload=yes;
   %let nameValue = "dirname=%sysfunc(pathname(&libname)) -- clientdownload=&clientdownload";
   %syscall insert_dataset(ProcessID,libname,memname,description,nameValue,rcp);
   %Checkrc(Insert_Dataset,_rc=&rcp);
%mend PackageInsertDataset;

%macro PackageInsertHtml(description, outpath, bodyFile, frameFile, contentsFile, clientdownload);
   filename b "&outpath.&bodyFile";
   filename f "&outpath.&frameFile";
   filename c "&outpath.&contentsFile";

   %let body=fileref:b;
   %let bodyUrl=&bodyFile;
   %let frame=fileref:f;
   %let frameUrl=&frameFile;
   %let contents=fileref:c;
   %let contentsUrl=&contentsFile;
   %let page=;
   %let pageUrl=;
   %if ^%length(&clientdownload) %then %let clientdownload=yes;
   %let nameValue="dirname=&outpath -- clientdownload=&clientdownload";

   %syscall insert_html(ProcessID, body, bodyUrl,
       frame, frameUrl,
       contents, contentsUrl,
       page, pageUrl,
       description, nameValue, rcp);

   %Checkrc(Insert_Html,_rc=&rcp);
%mend PackageInsertHtml;

/* in the macro below:

dependson is a list of files that the inserted file depends on.  this list must be  
delimited by commas and contain no spaces 

mimetype indicates the type of file, possible values are as follows, default is 
text/plain: 

application/msword 
application/xls 
application/octet-stream 
application/binary 
application/pdf 
application/postscript 
application/zip 
audio/basic 
image/jpeg 
image/gif 
image/tiff 
model/vrml 
text/html 
text/plain 
text/richtext 
text/xcmdlog 
video/mpeg 
video/quicktime 
*/
%macro PackageInsertFile(description,filepath,filename,dependson,mimetype,clientdownload);
   %let filenm = filename: &filepath.&filename;
   %let filetype = text;
   %if ^%length(&mimetype) %then %let mimetype = text/plain;
   %if ^%length(&clientdownload) %then %let clientdownload=yes;
   %let NameValue = dirname=&filepath -- TextFileName=&filename -- clientdownload=&clientdownload -- DependsOn=&dependson;
   %syscall insert_file(ProcessID,filenm,filetype,mimetype,description,NameValue,rcp);
   %Checkrc(Insert_File,_rc=&rcp);
%mend PackageInsertFile;

%macro PackagePublish(path,pubType);
   %if (&pubType = to_archive) %then %do; 
      %let properties = archive_path, archive_name;
   %end;
   %else %do;
      %let pubType = to_requester;
      %let properties = archive_path;
   %end;
   %let path = &OutPath;
   %let name = &ProcessName;
   %if (&pubType = to_archive) %then %do;
      %syscall package_publish(ProcessID, pubType, rcp, properties, path, name);
   %end;
   %else %do;
      %syscall package_publish(ProcessID, pubType, rcp, properties, path);
   %end;
   %Checkrc(Package_Publish,_rc=&rcp);
%mend PackagePublish;

%macro PackageEnd;
   %syscall package_end(ProcessID,rcp);
   %Checkrc(Package_End,_rc=&rcp);
   quit;
%mend PackageEnd;

* determine path and filename from a full reference;
%macro PathName(ref);
   %global TmpPath TmpName TmpName2; 
   %let error = 0;
   data _null_;
      l = length("&ref");
      if (l<2) then do;
         error "ERROR: Length of path reference is less than 2.";
	     call symput("exiterror",1);
	     stop;
      end;
      flag = 0;
      do i = 1 to l;
         if substr("&ref",i,1) in ('\','/') then x1 = i;
	     if substr("&ref",i,1) = '.' then do;
            x2 = i;
		    flag = 1;
	     end;
      end;
      if flag = 0 then x2 = l + 1;
      call symput('TmpPath',substr("&ref",1,x1));
      call symput('TmpName',substr("&ref",x1+1,x2-x1-1));
      call symput('TmpName2',substr("&ref",x1+1));
   run;
   %if (%length(&TmpPath)=0) or (%length(&TmpName)=0) %then %let exiterror = 1; 
%mend PathName;

* Check for naming convention;
%macro Naming(Dset=,nVars=,AllVars=);
   %let i = 1;
   %do %while(%length(%scan(&AllVars,&i)));
      %let Name&i = %scan(&AllVars,&i); 
      data tmp0;
         set &Dset;
		 length tmp $32;
         NCL1 = substr(trim(left(&&Name&i)),1,1);  
	     if upcase(NCL1) in ('_','A','B','C','D','E','F','G','H','I',
	                         'J','K','L','M','N','O','P','Q','R','S',
	 				         'T','U','V','W','X','Y','Z') then NCflag = 1;
		 if NCflag ne 1 then tmp = '_'||trim(left(&&Name&i));
		 else tmp = trim(left(&&Name&i));
	     order = _n_;
		 drop NCL1 NCFlag &&Name&i;
		 rename tmp=&&Name&i;
	  run;

	  proc sort data=tmp0;
	     by &&Name&i;
	  run;

      data tmp;
	     set tmp0;
	     keep &&Name&i;
	  run;

	  proc sort data=tmp out=tmp1 nodup;
	     by &&Name&i;
	  run;

      proc transpose data=tmp1 out=tmp2(drop=_name_);
         var &&Name&i; 
         id &&Name&i; 
      run;

	  proc contents data=tmp2 out=tmp3(keep=NAME) noprint;
      run; 

      %let found = 0; 
      data _null_;
	     set tmp3;
		 if upcase(NAME) = "_LABEL_" then do;
            call symput('found',1);
			stop;
		 end;
	  run; 

	  %if &found %then %do;
	     data tmp2;
		    set tmp2;
			drop _Label_;
		 run;
	  %end;

	  proc transpose data=tmp2 out=tmp4(rename=(col1=&&Name&i));
	     var _all_;
	  run;  

      proc sort data=tmp4;
	     by &&Name&i;
	  run;

      data tmp5;
         merge tmp0 tmp4;
	     by &&Name&i;
      run;

	  proc sort data=tmp5;
	     by order;
	  run;

	  data &Dset;
	     set tmp5;
	     &&Name&i = _name_;
	     drop order _name_;
	  run;
	  %let i = %eval(&i+1);
   %end;
   %put End of Checking Names in Data Set - &Dset;
%mend Naming;


%macro ImportScanRows(Flag=,ScanRows=);
%let workpath=%sysfunc(pathname(work));
%if &sysscp=WIN %then %let _slash=\;
%else %let _slash=/;

%if &ScanRows > 20 & &Flag %then %do;
   proc registry export="&workpath.&_slash.registry.txt";
   run;

   filename guessrow "&workpath.&_slash.guessingrows.txt";
   data _null_;
      file guessrow;
	  put '[\PRODUCTS\BASE\EFI]';
	  put "%str(%"GuessingRows%"=int:&ScanRows)";
   run;

   proc registry import=guessrow; 
   run; 

   %if %sysfunc(fexist(guessrow)) %then %let frc=%sysfunc(fdelete(guessrow));
%end;

filename registry "&workpath.&_slash.registry.txt";
%if %sysfunc(fexist(registry)) & &Flag=0 %then %do;
   proc registry import=registry;
   run;
   %let frc=%sysfunc(fdelete(registry));
%end;
%mend ImportScanRows;
